//
// ERGOT : The Windows MacTierra clone
// copyright (c) 2005 plushpuffin@gmail.com
// http://www.wwddfd.com/
//
// latest build: http://www.wwddfd.com/frell/ergot.zip
//
// licensing terms still under consideration...

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

namespace ERGOT
{
	/// <summary>
	/// Summary description for Census.
	/// </summary>
	public class Directory : System.Windows.Forms.Form
	{
		internal Machine ma = null;
		internal bool pauseUpdates = false;
		internal DateTime lastUpdate = DateTime.MinValue;
		private bool _isDisposed = false;
		private System.Windows.Forms.ImageList imgs_FDNormal;
		private System.Windows.Forms.ColumnHeader col_Species;
		private System.Windows.Forms.ColumnHeader col_Position;
		private System.Windows.Forms.ColumnHeader col_Children;
		private System.Windows.Forms.ListView lvDirectory;
		private System.ComponentModel.IContainer components;

		public Directory()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			this.DialogResult = DialogResult.None;
		}

		#region register Machine
        public void register( Machine newMa )
		{
			if( this.ma != null )
			{
				ma.MachineRefresh -= new MachineRefreshHandler(this.Machine_Report_Refresh_UI);
				ma.MachineStop -= new MachineRefreshHandler(this.Machine_Stop_Refresh_UI);
			}

			this.ma = null;
			species.Clear();
			lastUpdate = DateTime.MinValue;

			if( newMa == null ) { return; }

			ma = newMa;
			ma.MachineRefresh += new MachineRefreshHandler(this.Machine_Report_Refresh_UI);
			ma.MachineStop += new MachineRefreshHandler(this.Machine_Stop_Refresh_UI);

			updateDisplay();
		}
		#endregion

		#region Refresh/Update
		private void Machine_Report_Refresh_UI(object sender, MachineState m)
		{
			TimeSpan diff = DateTime.Now.Subtract(lastUpdate);
			if( diff.Seconds < 5 ) { return; }
			this.updateDisplay();
		}

		private void Machine_Stop_Refresh_UI(object sender, MachineState m)
		{
			this.updateDisplay();
		}

		public void updateDisplay()
		{
			if( pauseUpdates ) { return; }
			lock( lvDirectory ) 
			{
				lastUpdate = DateTime.Now;
				lvDirectory.BeginUpdate();
				lvDirectory.Items.Clear();

				ergot cur;
				ListViewItem lvItem;

				KTL.Node cur = ma.ergotList.Head;
			
				while( cur != null && ! cur.isNull )
				{
					cur = ((ergot)(cur.Value));
					lvItem = (ListViewItem)(species[curGBR.mySpeciesName]);
					if( lvItem == null )
					{
						lvItem = new ListViewItem();
						lvItem.ImageIndex = 2;
						lvItem.Tag = curGBR;
						lvGeneBank.Items.Add(lvItem);
						lvItem.Text = curGBR.mySpeciesName;
						species.Add(curGBR.mySpeciesName,lvItem);
						lvItem.SubItems.Add(curGBR.myCurrentPopulation.ToString().PadLeft(8,' '));
						lvItem.SubItems.Add(curGBR.myCount.ToString().PadLeft(8,' '));
						lvItem.SubItems.Add(curGBR.myPeak.ToString().PadLeft(8,' '));
					}
					cur = cur.NextNode;
				}
				lvDirectory.EndUpdate();
			}
		}

		#endregion

		#region Properties
		public bool isDisposed
		{
			get { return _isDisposed; }
		}
		#endregion

		#region Dispose
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( ma != null )
			{
				ma.MachineRefresh -= new MachineRefreshHandler(this.Machine_Report_Refresh_UI);
				ma.MachineStop -= new MachineRefreshHandler(this.Machine_Stop_Refresh_UI);
			}
			this.DialogResult = DialogResult.Abort;
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
			_isDisposed = true;
		}

		#endregion

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(Directory));
			this.lvDirectory = new System.Windows.Forms.ListView();
			this.col_Position = new System.Windows.Forms.ColumnHeader();
			this.col_Species = new System.Windows.Forms.ColumnHeader();
			this.col_Children = new System.Windows.Forms.ColumnHeader();
			this.imgs_FDNormal = new System.Windows.Forms.ImageList(this.components);
			this.SuspendLayout();
			// 
			// lvDirectory
			// 
			this.lvDirectory.Anchor = (((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right);
			this.lvDirectory.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																						  this.col_Position,
																						  this.col_Species,
																						  this.col_Children});
			this.lvDirectory.FullRowSelect = true;
			this.lvDirectory.GridLines = true;
			this.lvDirectory.LabelEdit = true;
			this.lvDirectory.LabelWrap = false;
			this.lvDirectory.LargeImageList = this.imgs_FDNormal;
			this.lvDirectory.Location = new System.Drawing.Point(8, 8);
			this.lvDirectory.MultiSelect = false;
			this.lvDirectory.Name = "lvDirectory";
			this.lvDirectory.Size = new System.Drawing.Size(376, 144);
			this.lvDirectory.SmallImageList = this.imgs_FDNormal;
			this.lvDirectory.TabIndex = 0;
			this.lvDirectory.View = System.Windows.Forms.View.Details;
			// 
			// col_Position
			// 
			this.col_Position.Text = "Position";
			// 
			// col_Species
			// 
			this.col_Species.Text = "Species";
			this.col_Species.Width = 120;
			// 
			// col_Children
			// 
			this.col_Children.Text = "Children";
			// 
			// imgs_FDNormal
			// 
			this.imgs_FDNormal.ColorDepth = System.Windows.Forms.ColorDepth.Depth32Bit;
			this.imgs_FDNormal.ImageSize = new System.Drawing.Size(16, 16);
			this.imgs_FDNormal.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imgs_FDNormal.ImageStream")));
			this.imgs_FDNormal.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// Directory
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(392, 173);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.lvDirectory});
			this.MinimumSize = new System.Drawing.Size(100, 64);
			this.Name = "Directory";
			this.ShowInTaskbar = false;
			this.SizeGripStyle = System.Windows.Forms.SizeGripStyle.Show;
			this.Text = "Directory";
			this.ResumeLayout(false);

		}
		#endregion
	}
}
