using System;
using System.Runtime.InteropServices;
using System.ComponentModel;
using System.Threading;

namespace Win32
{
	internal class HiPerfTimer
	{
		[DllImport("Kernel32.dll")]
		private static extern bool QueryPerformanceCounter(
			out long lpPerformanceCount);

		[DllImport("Kernel32.dll")]
		private static extern bool QueryPerformanceFrequency(
			out long lpFrequency);

		internal long startTime, stopTime;
		internal long freq;

		// Constructor
		public HiPerfTimer()
		{
			startTime = 0;
			stopTime  = 0;

			if (QueryPerformanceFrequency(out freq) == false)
			{
				// high-performance counter not supported
				throw new Win32Exception();
			}
		}

		// Start the timer
		public void Start()
		{
			//Thread.Sleep(0);
			QueryPerformanceCounter(out startTime);
		}

		// Stop the timer
		public void Stop()
		{
			QueryPerformanceCounter(out stopTime);
		}

		public double DurationSince
		{
			get
			{
				QueryPerformanceCounter(out stopTime);
				double elapsed = (double)( stopTime - startTime ) / (double)freq;
				startTime = stopTime;
				return elapsed;
			}
		}

		public double DurationTotal
		{
			get
			{
				QueryPerformanceCounter(out stopTime);
				double elapsed = (double)( stopTime - startTime ) / (double)freq;
				return elapsed;
			}
		}

		// Returns the duration of the timer (in seconds)
		public double Duration
		{
			get
			{
				return (double)(stopTime - startTime) / (double) freq;
			}
		}
	}
}