//
// ERGOT : The Windows MacTierra clone
// copyright (c) 2005 plushpuffin@gmail.com
// http://www.wwddfd.com/
//
// latest build: http://www.wwddfd.com/frell/ergot.zip
//
// licensing terms still under consideration...

using System;
using System.Drawing;
using System.Xml;
using System.ComponentModel;

namespace ERGOT
{
	#region delegates
	public delegate void MachineRefreshHandler(object sender, MachineState m);
	public delegate void MachineStepHandler(object sender, MachineThreadState e);
	#endregion

	#region event argument classes
	public class MachineState: EventArgs
	{
		private Bitmap _Snapshot;
		private ulong _amntWork, _numTicks, _numCycles;
		private int _Size, _Width, _Height, _Population, _NumSpecies, _Load;

		/// <summary>
		/// Create a new Machine event, to be used by some unknown user interface to display the Machine's updated statistics.
		/// </summary>
		/// <param name="mBitmap">The bitmap representation of the Machine's memory.</param>
		/// <param name="mTicks">The number of ticks that have elapsed.</param>
		/// <param name="mCycles">The number of slices that have elapsed.</param>
		/// <param name="mSize">The size of the Machine's memory space.</param>
		/// <param name="mWidth">The display width of the Machine's memory space.</param>
		/// <param name="mHeight">The display height of the Machine's memory space.</param>
		/// <param name="mPopulation">The number of active threads in the Machine.</param>
		/// <param name="mLoad">The number of bytes of memory currently in use by threads.</param>
		//public MachineState( System.Drawing.Bitmap mBitmap, uint mTicks, uint mCycles, int mSize, int mWidth, int mHeight, int mPopulation, int mLoad )
		
		public MachineState( Machine theMachine )
		{
			update(theMachine);
		}

		#region accessors
		public Bitmap Snapshot
		{
			get
			{
				return _Snapshot;
			}
		}

		public ulong Work
		{
			get
			{
				return _amntWork;
			}
		}

		public ulong Ticks
		{
			get
			{
				return _numTicks;
			}
		}

		public ulong Cycles
		{
			get
			{
				return _numCycles;
			}
		}

		public int Size
		{
			get
			{
				return _Size;
			}
		}

		public int Width
		{
			get
			{
				return _Width;
			}
		}

		public int Height
		{
			get
			{
				return _Height;
			}
		}

		public int Population
		{
			get
			{
				return _Population;
			}
		}

		public int NumSpecies
		{
			get
			{
				return _NumSpecies;
			}
		}

		public int Load
		{
			get
			{
				return _Load;
			}
		}

		#endregion

		#region Updater
		public void update( Machine theMachine )
		{
			this._Snapshot = theMachine.sbmp;
			this._amntWork = theMachine.amntWork;
			this._numTicks = theMachine.numTicks;
			this._numCycles = theMachine.numCycles;
			this._Size = theMachine.size;
			this._Width = theMachine.width;
			this._Height = theMachine.height;
			this._Population = theMachine.countAll;
			this._NumSpecies = theMachine.myGeneBank.Count;
			this._Load = theMachine.load;
		}
		#endregion
	}

	public class MachineThreadState: EventArgs
	{
		private ergot _Ergot;
		private ulong _Tick, _Cycle;
		private string _Instruction;
		/// <summary>
		/// Create a new Ergot event, to be used by some unknown user interface to display the results of a single tick.
		/// </summary>
		/// <param name="mErgot">The Ergot that was active for the indicated tick.</param>
		/// <param name="mTick">The indicated tick. Useful for synchronizing with MachineRefreshHandler events.</param>
		/// <param name="mCycle">The cycle of the indicated tick.</param>
		public MachineThreadState( ergot mErgot, ulong mTick, ulong mCycle, string mInstruction )
		{
			_Ergot = mErgot;
			_Tick = mTick;
			_Cycle = mCycle;
			_Instruction = mInstruction;
		}
		#region accessors
		public ergot Ergot
		{
			get
			{
				return _Ergot;
			}
		}

		public ulong Tick
		{
			get
			{
				return _Tick;
			}
		}

		public ulong Cycle
		{
			get
			{
				return _Cycle;
			}
		}

		public string Instruction
		{
			get
			{
				return _Instruction;
			}
		}
		#endregion
	}

	#endregion

	#region option interface class
	public class MachineOptions
	{
		public enum WritePermissions { Child, Cousin, Global }
		public enum AllocationAddress { AX, Parent, Random, RandomPacked }
		public enum ErrorType { Bit, Adjacent, Random, Stack }
		
		internal bool opDebug = false;

		//soup:
		internal int opSoupSizeRoot;
		internal int opSoupSliceTime, opSoupSaveAutoInterval;
		internal bool opSoupSliceConstant, opSoupSliceLeanBias, opSoupSaveAutoAsNew;
		internal double opSoupSliceFunctionOfSize;
		internal MachineOptions.WritePermissions opSoupWritePermissions;	// 0: child and unused, 1: child, 2: cousin, 3: global

		// reaper:
		internal double opReaperTrigger;
		internal bool opReaperErase, opReaperLog;

		// malloc/fork:
		internal MachineOptions.AllocationAddress opMallocation;	// 0:ax, 1:parent, 2:random, 3:random packed
		internal bool opMallocErase, opForkCopyRegisters;

		// errors:
		internal bool opErrFlagInstr;
		internal double opErrSlice, opErrCosmic, opErrCopy, opErrFlaw;
		internal MachineOptions.ErrorType opErrType;	// 0: bit, 1: adjacent, 2: random, 3: abs(st[sp] mod 32)

		public MachineOptions()
		{
			this.opDebug = false;
			this.opSoupSizeRoot = 8;
			this.opSoupSliceTime = 1024;
			this.opSoupSaveAutoInterval = 1000000;
			this.opSoupSliceConstant = false;
			this.opSoupSliceLeanBias = true;
			this.opSoupSaveAutoAsNew = false;
			this.opSoupSliceFunctionOfSize = 1.0;
			this.opSoupWritePermissions = MachineOptions.WritePermissions.Child;
			this.opReaperTrigger = 0.8;
			this.opReaperErase = false;
			this.opReaperLog = false;
			this.opMallocation = MachineOptions.AllocationAddress.AX;
			this.opMallocErase = false;
			this.opForkCopyRegisters = false;
			this.opErrSlice = 0.9;
			this.opErrCosmic = 0.002634;
			this.opErrCopy = 0.0002;
			this.opErrFlaw = 8.34E-05;
			this.opErrType = MachineOptions.ErrorType.Bit;
			this.opErrFlagInstr = false;
		}

		public MachineOptions( Machine ma )
		{
			this.opDebug = ma.opDebug;
			this.opSoupSizeRoot = ma.opSoupSizeRoot;
			this.opSoupSliceTime = ma.opSoupSliceTime;
			this.opSoupSaveAutoInterval = ma.opSoupSaveAutoInterval;
			this.opSoupSliceConstant = ma.opSoupSliceConstant;
			this.opSoupSliceLeanBias = ma.opSoupSliceLeanBias;
			this.opSoupSaveAutoAsNew = ma.opSoupSaveAutoAsNew;
			this.opSoupSliceFunctionOfSize = ma.opSoupSliceFunctionOfSize;
			this.opSoupWritePermissions = ma.opSoupWritePermissions;
			this.opReaperTrigger = ma.opReaperTrigger;
			this.opReaperErase = ma.opReaperErase;
			this.opReaperLog = ma.opReaperLog;
			this.opMallocation = ma.opMallocation;
			this.opMallocErase = ma.opMallocErase;
			this.opForkCopyRegisters = ma.opForkCopyRegisters;
			this.opErrSlice = ma.opErrSlice;
			this.opErrCosmic = ma.opErrCosmic;
			this.opErrCopy = ma.opErrCopy;
			this.opErrFlaw = ma.opErrFlaw;
			this.opErrType = ma.opErrType;
			this.opErrFlagInstr = ma.opErrFlagInstr;
		}

		public void ApplyOptions( Machine ma )
		{
			ma.opDebug = this.opDebug;
			if( ma.instrs == null )
			{
				ma.opSoupSizeRoot = (int)(this.opSoupSizeRoot);
			}
			ma.opSoupSliceTime = this.opSoupSliceTime;
			ma.opSoupSaveAutoInterval = this.opSoupSaveAutoInterval;
			ma.opSoupSliceConstant = this.opSoupSliceConstant;
			ma.opSoupSliceLeanBias = this.opSoupSliceLeanBias;
			ma.opSoupSaveAutoAsNew = this.opSoupSaveAutoAsNew;
			ma.opSoupSliceFunctionOfSize = this.opSoupSliceFunctionOfSize;
			ma.opSoupWritePermissions = this.opSoupWritePermissions;
			ma.opReaperTrigger = this.opReaperTrigger;
			ma.opReaperErase = this.opReaperErase;
			ma.opReaperLog = this.opReaperLog;
			ma.opMallocation = this.opMallocation;
			ma.opMallocErase = this.opMallocErase;
			ma.opForkCopyRegisters = this.opForkCopyRegisters;
			ma.opErrSlice = this.opErrSlice;
			ma.opErrCosmic = this.opErrCosmic;
			ma.opErrCopy = this.opErrCopy;
			ma.opErrFlaw = this.opErrFlaw;
			ma.opErrType = this.opErrType;
			ma.opErrFlagInstr = this.opErrFlagInstr;
		}

		[DescriptionAttribute("Debug mode enables various debugging features which may or may not exist."),
		CategoryAttribute("General")]
		public bool Debug
		{
			get
			{
				return opDebug;
			}

			set
			{
				opDebug = value;
			}
		}

		[DescriptionAttribute("The number of cells in the soup is: 2^(2*SoupSizeRoot)."),
		CategoryAttribute("Soup")]
		public int SoupSizeRoot
		{
			get
			{
				return opSoupSizeRoot;
			}
		}

		[DescriptionAttribute("Soup Slice Time (ticks)"),
		CategoryAttribute("Soup")]
		public int SoupSliceTime
		{
			get
			{
				return opSoupSliceTime;
			}

			set
			{
				opSoupSliceTime = value;
			}
		}

		[DescriptionAttribute("Soup Auto-Save Interval (cycles)"),
		CategoryAttribute("Soup")]
		public int SoupSaveAutoInterval
		{
			get
			{
				return opSoupSaveAutoInterval;
			}

			set
			{
				opSoupSaveAutoInterval = value;
			}
		}

		[DescriptionAttribute("Constant Soup Processor Slices?"),
		CategoryAttribute("Soup")]
		public bool SoupSliceConstant
		{
			get
			{
				return opSoupSliceConstant;
			}

			set
			{
				opSoupSliceConstant = value;
			}
		}

		[DescriptionAttribute("Soup Processor Slices: Bias for lean creatures?"),
		CategoryAttribute("Soup")]
		public bool SoupSliceLeanBias
		{
			get
			{
				return opSoupSliceLeanBias;
			}

			set
			{
				opSoupSliceLeanBias = value;
			}
		}
		
		[DescriptionAttribute("Soup Auto-Save as New?"),
		CategoryAttribute("Soup")]
		public bool SoupSaveAutoAsNew
		{
			get
			{
				return opSoupSaveAutoAsNew;
			}

			set
			{
				opSoupSaveAutoAsNew = value;
			}
		}

		[DescriptionAttribute("Soup Processor Slices: Function of Size"),
		CategoryAttribute("Soup")]
		public double SoupSliceFunctionOfSize
		{
			get
			{
				return opSoupSliceFunctionOfSize;
			}

			set
			{
				opSoupSliceFunctionOfSize = value;
			}
		}
		
		[DescriptionAttribute("Soup Write Permissions."),
		CategoryAttribute("Soup")]
		public MachineOptions.WritePermissions SoupWritePermissions
		{
			get
			{
				return opSoupWritePermissions;
			}

			set
			{
				opSoupWritePermissions = value;
			}
		}
		
		[DescriptionAttribute("Reaper Trigger (percentage full)"),
		CategoryAttribute("Reaper")]
		public double ReaperTrigger
		{
			get
			{
				return opReaperTrigger;
			}

			set
			{
				opReaperTrigger = value;
			}
		}

		[DescriptionAttribute("Reaper Erase on kill?"),
		CategoryAttribute("Reaper")]
		public bool ReaperErase
		{
			get
			{
				return opReaperErase;
			}

			set
			{
				opReaperErase = value;
			}
		}

		[DescriptionAttribute("Reaper logs kills?"),
		CategoryAttribute("Reaper")]
		public bool ReaperLog
		{
			get
			{
				return opReaperLog;
			}

			set
			{
				opReaperLog = value;
			}
		}

		[DescriptionAttribute("New creature allocation position"),
		CategoryAttribute("Allocation")]
		public MachineOptions.AllocationAddress Mallocation
		{
			get
			{
				return opMallocation;
			}

			set
			{
				opMallocation = value;
			}
		}
		
		[DescriptionAttribute("Erase on new creature allocation?"),
		CategoryAttribute("Allocation")]
		public bool MallocErase
		{
			get
			{
				return opMallocErase;
			}

			set
			{
				opMallocErase = value;
			}
		}
		
		[DescriptionAttribute("Copy registers on new creature fork?"),
		CategoryAttribute("Allocation")]
		public bool ForkCopyRegisters
		{
			get
			{
				return opForkCopyRegisters;
			}

			set
			{
				opForkCopyRegisters = value;
			}
		}
		
		[DescriptionAttribute("Slice Allocation Variation"),
		CategoryAttribute("Errors")]
		public double ErrSlice
		{
			get
			{
				return opErrSlice;
			}

			set
			{
				opErrSlice = value;
			}
		}

		[DescriptionAttribute("Cosmic Radiation Frequency"),
		CategoryAttribute("Errors")]
		public double ErrCosmic
		{
			get
			{
				return opErrCosmic;
			}

			set
			{
				opErrCosmic = value;
			}
		}
		
		[DescriptionAttribute("Copy Error Frequency"),
		CategoryAttribute("Errors")]
		public double ErrCopy
		{
			get
			{
				return opErrCopy;
			}

			set
			{
				opErrCopy = value;
			}
		}

		[DescriptionAttribute("Instruction Read Error Frequency"),
		CategoryAttribute("Errors")]
		public double ErrFlaw
		{
			get
			{
				return opErrFlaw;
			}

			set
			{
				opErrFlaw = value;
			}
		}

		[DescriptionAttribute("Error Type"),
		CategoryAttribute("Errors")]
		public MachineOptions.ErrorType ErrType
		{
			get
			{
				return opErrType;
			}

			set
			{
				opErrType = value;
			}
		}

		[DescriptionAttribute("Flag Instruction Sets Flag & Consumes Cycles"),
		CategoryAttribute("Errors")]
		public bool FlagInstr
		{
			get
			{
				return opErrFlagInstr;
			}

			set
			{
				opErrFlagInstr = value;
			}
		}
	}
	#endregion

	/// <summary>
	/// A Virtual Machine which executes hundreds of threads in a memory space.
	/// The machine is deliberately flawed in order to give the threads a chance to mutate.
	/// </summary>
	public class Machine
	{
		#region Member Constants
		public const short	c_rel_max			= (1 << 15) - 1;
		public const short	c_rel_min			= (1 << 15) * -1;
		public const byte	c_cell_dead			= 0;
		public const byte	c_cell_living		= 1;
		public const byte	c_cell_executed		= 2;
		public const byte	c_cell_embryo		= 3;
		public const byte	c_cell_embwri		= 4;
		public const short	c_instr_space		= 6;
		public const int	c_num_instrs		= 64;
		public const int	c_slicevary_size	= 23;
		public const int	c_ress_q_size		= 512;
		public const string c_default_creature	= "010101010B0305053A3800000000113937000000011B133A010100013D34000001013E31000001002B01010000232425010001003C202B31000100001B1D31000100012B0100010129282735010101002B";
		#endregion

		#region Member Variables

		// events
		public event MachineRefreshHandler MachineReport;
		public event MachineRefreshHandler MachineRefresh;
		public event MachineRefreshHandler MachineStop;
		public event MachineStepHandler MachineSingleStep;
		private MachineState mast = null;

		// simple state variables
		internal int size, sizeplusone, load, countAdult, countAll, width, height;
		internal int modu;
		internal ulong amntWork, numTicks, numCycles;
		public bool isCycling, mustStopCycling;
		internal KTL.Node stepCursor = null;
		internal int resurrectionStack_TopIndex = -1;
		internal uint numSpeciesSighted = 0;
		public byte soupDisplayMode = 0;	// 0 for instr-color, 1 for adult/children.
		// TODO: 2 for fecundity, 3 for by-species.

		//
		// User-modifiable settings:
		//
		internal bool opDebug = false;

		//soup:
		internal int opSoupSizeRoot;
		internal int opSoupSliceTime, opSoupSaveAutoInterval;
		internal bool opSoupSliceConstant, opSoupSliceLeanBias, opSoupSaveAutoAsNew;
		internal double opSoupSliceFunctionOfSize;
		internal MachineOptions.WritePermissions opSoupWritePermissions;	// 0: child and unused, 1: child, 2: cousin, 3: global

		// reaper:
		internal double opReaperTrigger;
		internal bool opReaperErase, opReaperLog;

		// malloc/fork:
		internal MachineOptions.AllocationAddress opMallocation;	// 0:ax, 1:parent, 2:random, 3:random packed
		internal bool opMallocErase, opForkCopyRegisters;

		// errors:
		internal bool opErrFlagInstr;
		internal double opErrSlice, opErrCosmic, opErrCopy, opErrFlaw;
		internal MachineOptions.ErrorType opErrType;	// 0: bit, 1: adjacent, 2: random, 3: abs(st[sp] mod 32)

		//
		// End User-modifiable settings.
		//

		// representations of soup
		internal Bitmap sbmp;
		internal byte[] soup;
		internal byte[] usage;
		public char[] hexinstrs = null;

		// drawing-related
		internal Rectangle soupRect;
		internal System.Drawing.Pen p_internal = new System.Drawing.Pen( System.Drawing.Color.HotPink, 1 );
		internal System.Drawing.Pen p_external = new System.Drawing.Pen( System.Drawing.Color.Yellow, 1 );

		// lists
		internal KTL.dList ergotList;
		internal KTL.dList reaperQueue;
		
		// various collections
		//		public System.Collections.Hashtable geneBank;
		//		public System.Collections.Hashtable speciesRegistry;
		public GeneBank myGeneBank = null;
		public byte[,] cellshades;
		public byte[,] deadshades;
		public string[] instrs;
		public bool[] istt;
		internal ergot[] resurrectionStack;
		
		//internal System.Collections.Hashtable FSMCache;
		internal byte[][][,] FSMLookup;
		// by length of template, by value of template -> gets you the FSM

		// precompute some slice variations so we don't have to constantly
		// do multiplication to vary the slice size.
		public int[] sliceVary = null;
		
		// random number generator
		public Random wire = null;
		#endregion

		#region XML Import/Export Variables
		private int _errorCode = 0;
		
		private string _xmlFileName;
		private System.Text.StringBuilder _errorString = null;
		
		public readonly System.Globalization.NumberStyles sharedStyle;
		public readonly System.Globalization.NumberStyles intStyle;
		public readonly System.Globalization.NumberStyles hexStyle;
		public readonly System.Globalization.NumberStyles dblStyle;
		public readonly System.Globalization.NumberStyles bytStyle;
		#endregion

		#region random number initializer
		/// <summary>
		/// Initialize the random number generator.
		/// You're free to use this if you wish, but you can just as easily call wire.NextWhatever()
		/// The recommended strategy is to call it once at the beginning of your function to make
		/// sure that it's instantiated, and then just use wire directly from that point on.
		/// </summary>
		/// <returns>a random double-width floating point number.</returns>
		public double getRandom()
		{
			double d;
			if( wire == null) 
			{
				wire = new Random();
			}
			lock (wire) 
			{
				d = wire.NextDouble();
			}
			return d;
		}
		#endregion

		#region loadDefaults
		/// <summary>
		/// Set default variables for the Machine. Called automatically by the Machine's constructor.
		/// </summary>
		public void loadDefaults()
		{
			if( instrs != null )
			{
				// no need to loadDefaults() again...
				return;
			}

			MachineOptions defaults = new MachineOptions();
			defaults.ApplyOptions(this);
			
			initializeCellShades();
			
			instrs = new String[c_num_instrs];
			for( int i = 0; i < c_num_instrs; ++i )
			{
				instrs[i] = "0x" + i.ToString("X2");
			}

			instrs[0x00] = "nop_0";
			instrs[0x01] = "nop_1";
			instrs[0x02] = "nop_2";
			instrs[0x03] = "xor1";
			instrs[0x04] = "xor2";
			instrs[0x05] = "shclo";
			instrs[0x06] = "shcro";
			instrs[0x07] = "shdlo";
			instrs[0x08] = "shdro";
			instrs[0x09] = "azero";
			instrs[0x0a] = "bzero";
			instrs[0x0b] = "czero";
			instrs[0x0c] = "dzero";
			instrs[0x0d] = "swapab";
			instrs[0x0e] = "swapcd";
			instrs[0x0f] = "xorab_c";
			instrs[0x10] = "addac_a";
			instrs[0x11] = "subac_a";
			instrs[0x12] = "addab_c";
			instrs[0x13] = "subab_c";
			instrs[0x14] = "addac_c";
			instrs[0x15] = "subac_c";
			instrs[0x16] = "addbc_c";
			instrs[0x17] = "subbc_c";
			instrs[0x18] = "slt_ab_c";
			instrs[0x19] = "seq_ab_c";
			instrs[0x1a] = "sne_ab_c";
			instrs[0x1b] = "inc_a";
			instrs[0x1c] = "dec_a";
			instrs[0x1d] = "inc_b";
			instrs[0x1e] = "dec_b";
			instrs[0x1f] = "inc_c";
			instrs[0x20] = "dec_c";
			instrs[0x21] = "inc_d";
			instrs[0x22] = "dec_d";
			instrs[0x23] = "push_ax";
			instrs[0x24] = "push_bx";
			instrs[0x25] = "push_cx";
			instrs[0x26] = "push_dx";
			instrs[0x27] = "pop_ax";
			instrs[0x28] = "pop_bx";
			instrs[0x29] = "pop_cx";
			instrs[0x2a] = "pop_dx";
			instrs[0x2b] = "if_cz";
			instrs[0x2c] = "if_cz4";
			instrs[0x2d] = "if_cz8";
			instrs[0x2e] = "if_dz";
			instrs[0x2f] = "if_dz4";
			instrs[0x30] = "if_dz8";
			instrs[0x31] = "jump";
			instrs[0x32] = "jmpf";
			instrs[0x33] = "jmpb";
			instrs[0x34] = "call";
			instrs[0x35] = "ret";
			instrs[0x36] = "addr";
			instrs[0x37] = "adrf";
			instrs[0x38] = "adrb";
			instrs[0x39] = "mov_ab";
			instrs[0x3a] = "mov_cd";
			instrs[0x3b] = "mov_ipc";
			instrs[0x3c] = "mov_iab";
			instrs[0x3d] = "malloc";
			instrs[0x3e] = "fork";
			instrs[0x3f] = "flag";
			
			istt = new bool[c_num_instrs];
			istt[0x00] = true;
			istt[0x01] = true;
			istt[0x02] = false;
			istt[0x03] = true;
			istt[0x04] = false;
			istt[0x05] = true;
			istt[0x06] = false;
			istt[0x07] = false;
			istt[0x08] = false;
			istt[0x09] = false;
			istt[0x0a] = false;
			istt[0x0b] = true;
			istt[0x0c] = false;
			istt[0x0d] = false;
			istt[0x0e] = false;
			istt[0x0f] = false;
			istt[0x10] = false;
			istt[0x11] = true;
			istt[0x12] = false;
			istt[0x13] = true;
			istt[0x14] = false;
			istt[0x15] = false;
			istt[0x16] = false;
			istt[0x17] = false;
			istt[0x18] = false;
			istt[0x19] = false;
			istt[0x1a] = false;
			istt[0x1b] = true;
			istt[0x1c] = false;
			istt[0x1d] = true;
			istt[0x1e] = false;
			istt[0x1f] = true;
			istt[0x20] = true;
			istt[0x21] = false;
			istt[0x22] = false;
			istt[0x23] = true;
			istt[0x24] = true;
			istt[0x25] = true;
			istt[0x26] = true;
			istt[0x27] = true;
			istt[0x28] = true;
			istt[0x29] = true;
			istt[0x2a] = true;
			istt[0x2b] = true;
			istt[0x2c] = false;
			istt[0x2d] = false;
			istt[0x2e] = false;
			istt[0x2f] = false;
			istt[0x30] = false;
			istt[0x31] = true;
			istt[0x32] = false;
			istt[0x33] = true;
			istt[0x34] = true;
			istt[0x35] = true;
			istt[0x36] = true;
			istt[0x37] = true;
			istt[0x38] = true;
			istt[0x39] = true;
			istt[0x3a] = true;
			istt[0x3b] = false;
			istt[0x3c] = true;
			istt[0x3d] = true;
			istt[0x3e] = true;
			istt[0x3f] = false;

			_errorString = new System.Text.StringBuilder(2001);
		}
		#endregion

		#region Constructor and Destructor
		/// <summary>
		/// Create a new machine. Set default values.
		/// WARNING: You must call initialize() before you begin using the Machine!
		/// </summary>
		public Machine()
		{
			sharedStyle = System.Globalization.NumberStyles.AllowLeadingWhite | System.Globalization.NumberStyles.AllowTrailingWhite;
			intStyle = sharedStyle | System.Globalization.NumberStyles.AllowThousands | System.Globalization.NumberStyles.AllowLeadingSign;
			hexStyle = sharedStyle | System.Globalization.NumberStyles.AllowHexSpecifier;
			dblStyle = sharedStyle | System.Globalization.NumberStyles.AllowThousands | System.Globalization.NumberStyles.AllowLeadingSign | System.Globalization.NumberStyles.AllowDecimalPoint | System.Globalization.NumberStyles.AllowExponent;
			bytStyle = sharedStyle | System.Globalization.NumberStyles.AllowLeadingSign;
			loadDefaults();
		}

		public Machine( string xmlFileName )
		{
			sharedStyle = System.Globalization.NumberStyles.AllowLeadingWhite | System.Globalization.NumberStyles.AllowTrailingWhite;
			intStyle = sharedStyle | System.Globalization.NumberStyles.AllowThousands | System.Globalization.NumberStyles.AllowLeadingSign;
			hexStyle = sharedStyle | System.Globalization.NumberStyles.AllowHexSpecifier;
			dblStyle = sharedStyle | System.Globalization.NumberStyles.AllowThousands | System.Globalization.NumberStyles.AllowLeadingSign | System.Globalization.NumberStyles.AllowDecimalPoint | System.Globalization.NumberStyles.AllowExponent;
			bytStyle = sharedStyle | System.Globalization.NumberStyles.AllowLeadingSign;
			loadDefaults();
			this.FromXMLFile( xmlFileName );
		}
		
		public void checkStatus()
		{
			if( ! this.isValid )
			{
				throw new Exception("The Machine's state is not valid! " + this._errorString.ToString() + ".");
			}
		}

		public void initialize()
		{
			// set error code to zero
			_errorCode = 0;
			_errorString = new System.Text.StringBuilder(2001);

			// create defaults
			if( opSoupSizeRoot < 7 )
			{
				opSoupSizeRoot = 7;
			}
			else if( opSoupSizeRoot > 15 )
			{
				opSoupSizeRoot = 15;
			}
			size = 1 << ((int)opSoupSizeRoot << 1);
			sizeplusone = size + 1;
			height = width = 1 << (int)opSoupSizeRoot;
			modu = size - 1;

			load = 0;
			countAll = 0;
			countAdult = 0;
			amntWork = 0;
			numTicks = 0;
			numCycles = 0;
			
			try
			{
				// create soup
				soup = new Byte[size];
				// create soup usage bitvec
				usage = new Byte[size];

				for( int i = 0; i < size; ++i )
				{
					soup[i] = 0;
					usage[i] = Machine.c_cell_dead;
				}

				// create empty bitmap representation of soup
				sbmp = new Bitmap(width, height, System.Drawing.Imaging.PixelFormat.Format24bppRgb);
				soupRect = new Rectangle(0,0,width,height);

				// create empty ergot list
				ergotList = new KTL.dList();

				// create empty reaper queue
				reaperQueue = new KTL.dList();

				// create empty GeneBank
				myGeneBank = new GeneBank();

				// create the Finite State Machine cache...
				//FSMCache = new System.Collections.Hashtable(4001);

				if( this.FSMLookup == null )
				{
					// create the FSM Lookup table.
					this.FSMLookup = new Byte[9][][,];
					// we're going to leave index zero null because there are no patterns of length zero.
					FSMLookup[0] = null;
					for( int i = 1; i < 9; ++i )
					{
						FSMLookup[i] = new Byte[(1<<i)][,];
					}
				}

				if( this.sliceVary == null )
				{
					sliceVary = new int[c_slicevary_size];
				}
				double slicePrecision = getRandom();
				for( int i = 0; i < c_slicevary_size; ++i )
				{
					slicePrecision = 1.0 + (((wire.NextDouble() - 0.5) * opErrSlice) * 2.0);
					sliceVary[i] = (int)(opSoupSliceTime * slicePrecision);
				}

				// create the resurrectionStack for re-using dead creatures
				// (to avoid the overhead of creating new objects)
				resurrectionStack = new ergot[c_ress_q_size];

				hexinstrs = new char[c_num_instrs << 1];
				string shex;
				for( int i = 0; i < c_num_instrs; ++i )
				{
					shex = i.ToString("X2");
					this.hexinstrs[i << 1] = shex[0];
					this.hexinstrs[(i << 1)+1] = shex[1];
				}

				mast = new MachineState(this);

			}
			catch( System.OutOfMemoryException ex )
			{
				this._errorCode = 666;
				this._errorString.Append( "Ran out of memory during initialization: " + ex.Message + "\n");
			}
			catch( Exception ex )
			{
				this._errorCode = 777;
				this._errorString.Append( "Exception during initialization: " + ex.Message + "\n" );
			}

			// do a garbage cleanup to get rid of anything I just lost track of...
			GC.Collect();
		}
		

		/// <summary>
		/// Release this Machine's bitmap resources upon destruction.
		/// </summary>
		~Machine()
		{
			if( sbmp != null ) 
			{
				sbmp.Dispose();
				sbmp = null;
			}
			GC.Collect();
		}
		#endregion

		#region Properties
		public bool isValid
		{
			get
			{
				return (_errorCode < 1) && (soup != null) && (usage != null);
			}
		}

		public int ErrorCode
		{
			get 
			{
				return _errorCode;
			}
		}

		public string ErrorString
		{
			get
			{
				return _errorString.ToString();
			}
		}
		#endregion

		#region Event Triggers
		internal void onReportStatus()
		{
			if( MachineReport != null )
			{
				mast.update(this);
				MachineReport(this, mast);
			}
		}

		internal void onRefresh()
		{
			if( MachineRefresh != null )
			{
				updateSoupBitmap();
				mast.update(this);
				MachineRefresh(this, mast);
			}
		}

		internal void onStop()
		{
			GC.Collect();
			if( MachineStop != null )
			{
				updateSoupBitmap();
				MachineStop(this, new MachineState(this));
			}
		}

		internal void onStep(MachineThreadState e)
		{
			if( MachineSingleStep != null )
			{
				updateSoupBitmap();
				MachineSingleStep(this, e);
			}
		}
		#endregion

		#region add Ergot to Soup
		/// <summary>
		/// Add an ergot from a file to a specific place in the soup.
		/// </summary>
		/// <param name="fileName">File name.</param>
		/// <param name="desiredPosition">Desired position for the new creature.</param>
		public void addErgotFromFile( string fileName, int desiredPosition )
		{
			System.Collections.ArrayList codeBuffer = new System.Collections.ArrayList(128);

			System.IO.StreamReader SR;
			string buffer;
			SR=System.IO.File.OpenText(fileName);
			buffer=SR.ReadLine();
			while(buffer!=null)
			{
				if( System.Text.RegularExpressions.Regex.IsMatch(buffer, "\\A[A-Fa-f0-9]{2}\\Z" ) )
				{
					codeBuffer.Add(buffer);
				}
				buffer=SR.ReadLine();
			}
			SR.Close();

			short length = (short)codeBuffer.Count;
			byte[] code = new byte[length];
			for( int i = 0; i < length; ++i )
			{
				code[i] = byte.Parse( (string)codeBuffer[i], System.Globalization.NumberStyles.AllowHexSpecifier );
			}
			string theSpecies = length.ToString() + "/0";
			addErgot( theSpecies, code, desiredPosition, length );
		}


		/// <summary>
		/// Add a creature represented as a character string.
		/// </summary>
		/// <param name="ergotSpecies">Species name to be used.</param>
		/// <param name="ergotAssembly">String containing the assembly, as hex-encoded characters.</param>
		/// <param name="desiredPosition">Desired position for the new creature.</param>
		public void addErgot( string ergotSpecies, string ergotAssembly, int desiredPosition )
		{
			
			short length = (short)ergotAssembly.Length;
			// Here we create the assembly source byte vector for the new ergot.
			byte[] code = new byte[ length ];
			for( short i = 0; i < length - 1; ++i ) 
			{
				code[i] = (byte)ergotAssembly[i];
			}
			addErgot( ergotSpecies, code, desiredPosition, length );
		}


		/// <summary>
		/// Add an ergot represented in the native format of an array of bytes.
		/// </summary>
		/// <param name="ergotSpecies">Species name to be used.</param>
		/// <param name="code">The byte[] array.</param>
		/// <param name="desiredPosition">Desired position for the new creature.</param>
		/// <param name="length">Length of the byte[] array.</param>
		public void addErgot( string ergotSpecies, byte[] code, int desiredPosition, short length )
		{
			checkStatus();
			ergot adam = findEmptySpace( desiredPosition, 0, length );
			if( adam == null )
			{
				// no empty space found.
				return;
			}
			int index = 0;
			for( int i = adam.pos; index < adam.mySize; i = deprecated_wrap(i+1) ) 
			{
				soup[i] = code[index];
				++index;
			}
			adam.fork(null);				// This ergot is divine. It has no parent.
			//adam.myGeneBankRecord.mySpeciesName = ergotSpecies;
			myGeneBank.renameSpecies(adam.myGeneBankRecord.mySpeciesName,ergotSpecies);
			if( ! this.isCycling )
			{
				this.onStop();
			}
		}
		#endregion

		#region runErgots
		/// <summary>
		/// Run for 4 million cycles. Update infrequently.
		/// </summary>
		public void runErgots()
		{
			runErgots( 10, 50, 4000000 );
		}

		public void runErgotsOneHundredCycles()
		{
			runErgots( 10, 50, 100 );
		}

		/// <summary>
		/// Run one creature for one tick.
		/// This function will keep track of which creature it executed last,
		/// so the next time you call it, it will run the next creature in the list.
		/// </summary>
		public void runOneErgotOneTick()
		{
			if( isCycling )
			{
				// don't run if we're already running.
				return;
			}
			checkStatus();
			getRandom();
			double mySize = size;
			ergot curErgot;
			isCycling = true;

			if( stepCursor == null || stepCursor.isNull )
			{
				// initialize stepCursor
				stepCursor = ergotList.Head;
			}
			if( stepCursor.isNull )
			{
				// no creatures in the soup!
				isCycling = false;
				return;
			}

			// now do stuff that's basically an un-looped copy of runErgots()
			curErgot = (ergot)stepCursor.Value;
			if( curErgot.isAlive )
			{
				curErgot.runFor( 1 );
				// since processor debt is not decremented in runOneErgotOneTick(),
				// we must make sure it isn't incremented, either.
				curErgot.processorDebt = 0;
				numTicks += 1;
			}

			if( stepCursor != null )
			{
				stepCursor = stepCursor.NextNode;
			}

			if( stepCursor == null || stepCursor.isNull )
			{
				// only increment numCycles if we just executed the last ergot in the list.
				// same for culling the herd - we don't want our cursor to become corrupted.
				++numCycles;
				if( (((double)load)/mySize) > opReaperTrigger )
				{
					cullTheHerd();
				}
				stepCursor = null;
			}
			if( wire.NextDouble() < opErrCosmic )
			{
				// do something strange to a random instruction.
				// what we do is XOR it with either 16, 8, 4, 2, or 1.
				// essentially, we're flipping a random bit.
				soup[ wire.Next(size) ] ^= (byte)(1 << wire.Next(c_instr_space));
			}
			isCycling = false;
			GC.Collect();
			this.onStep(new MachineThreadState(curErgot, numTicks, numCycles,
				instrs[ soup[
				deprecated_wrap(curErgot.ip+curErgot.pos)
				]
				]
				)
				);
		}


		/// <summary>
		/// Executes the Machine's thread manager, which parcels out cycles to every living creature.
		/// </summary>
		/// <param name="updateEveryXCycles">How often should the soup view be updated?</param>
		/// <param name="runForXCycles">How many cycles should the Machine run for before it stops?</param>
		public void runErgots( int updateStatusEveryXCycles, int updatePictureEveryXCycles, int runForXCycles )
		{
			if( isCycling )
			{
				// don't run if we're already running.
				return;
			}
			checkStatus();
			double slicePrecision = getRandom();
			double mySize = size;
			double mySizeDeviation, mySizeBias;
			int sliceAllocation, adiff;
			KTL.Node cur;
			ergot curErgot;
			int cyclesSinceStatusUpdate = 0;
			int cyclesSincePictureUpdate = 0;
			isCycling = true;
			for( int cycleNum = 0; cycleNum < runForXCycles && ! mustStopCycling; ++cycleNum )
			{
				cur = ergotList.Head;
				if( cur.isNull )
				{
					// no creatures in the soup!
					break;
				}
                while( ! cur.isNull )
				{
					curErgot = (ergot)cur.Value;
					if( curErgot.isAlive )
					{
						if( opSoupSliceConstant )
						{
							sliceAllocation = (int)(sliceVary[wire.Next(c_slicevary_size-1)] * curErgot.mySlicerBias);
						}
						else
						{
							// size-preferential slicing.
							mySizeDeviation = curErgot.mySize / 100.5;
							mySizeBias = Math.Pow(mySizeDeviation, (opSoupSliceFunctionOfSize));
							mySizeBias /= mySizeDeviation;
							mySizeBias = mySizeBias + (2.2*(mySizeBias-1.0));
							if( mySizeBias < 0.1 ) { mySizeBias = 0.1; }
							sliceAllocation = (int)(sliceVary[wire.Next(c_slicevary_size-1)] * curErgot.mySlicerBias * mySizeBias);
						}
						if( curErgot.processorDebt == 0 )
						{
							// if there is no processorDebt, run normally.
							curErgot.runFor( sliceAllocation );
						}
						else
						{
							// if there is processorDebt, we must run for a shorter time.
							// (or not at all if there's a large debt)
							adiff = sliceAllocation - curErgot.processorDebt;
							if( curErgot.processorDebt < 0 )
							{
								// we have an allocation bonus!
								curErgot.processorDebt = 0;
								curErgot.runFor( adiff );
							}
							else if( adiff > 0 )
							{
								// run for whatever allocation remains.
								amntWork += (uint)(curErgot.processorDebt);
								curErgot.processorDebt -= sliceAllocation;
								curErgot.runFor( adiff );
							}
							else
							{
								amntWork += (uint)sliceAllocation;
								curErgot.processorDebt -= sliceAllocation;
							}
						}
					}
					cur = cur.NextNode;
				} // while
				if( wire.NextDouble() < opErrCosmic )
				{
					// do something strange to a random instruction.
					soup[ wire.Next(size) ] = (byte)wire.Next(c_num_instrs);
				}
				if( (((double)load)/mySize) > opReaperTrigger )
				{
					cullTheHerd();
				}
				++numCycles;
				++cyclesSinceStatusUpdate;
				++cyclesSincePictureUpdate;
				if( cyclesSinceStatusUpdate > updateStatusEveryXCycles )
				{
					this.onReportStatus();
					cyclesSinceStatusUpdate = 0;
				}
				if( cyclesSincePictureUpdate > updatePictureEveryXCycles )
				{
					//					GC.Collect();
					this.onRefresh();
					cyclesSincePictureUpdate = 0;
				}
			} // for
			isCycling = false;
			this.onStop();
		}
		#endregion

		#region The Reaper
		/// <summary>
		/// When the soup is too full, this function is called in order to wipe out
		/// the creatures which raise the most errors. (those that are highest in the reaper queue)
		/// </summary>
		public void cullTheHerd()
		{
			KTL.Node nex = reaperQueue.Head;
			KTL.Node cur;
			int numCulled = 0;
			ergot curErgot;
			int numToCull = (int)(countAdult * opReaperTrigger * 0.9 );
			while( numCulled < numToCull && ! nex.isNull )
			{
				cur = nex;
				nex = nex.NextNode;
				curErgot = (ergot)cur.Value;

				// pay attention! if this node has an embryo in development, then
				// we're actually killing TWO creatures!
				if( curErgot.myChild != null ) { ++numCulled; }
				curErgot.kill();
				++numCulled;
				// The ergot handles its death and removes its node from the reaper queue.
				// That's why we get the next node before we call kill()
				// I could probably call .NextNode on the removed node, but I don't
				// want to take any chances, especially if I modify dList in the future.
			}
		}

		
		public void clearTheHerd()
		{
			KTL.Node nex = reaperQueue.Head;
			KTL.Node cur;
			int numCulled = 0;
			ergot curErgot;
			int numToCull = (int)(countAll);
			while( numCulled < numToCull && ! nex.isNull )
			{
				cur = nex;
				nex = nex.NextNode;
				curErgot = (ergot)cur.Value;

				// pay attention! if this node has an embryo in development, then
				// we're actually killing TWO creatures!
				if( curErgot.myChild != null ) { ++numCulled; }
				curErgot.kill();
				++numCulled;
				// The ergot handles its death and removes its node from the reaper queue.
				// That's why we get the next node before we call kill()
				// I could probably call .NextNode on the removed node, but I don't
				// want to take any chances, especially if I modify dList in the future.
			}
		}
		#endregion

		#region pattern searching
		/// <summary>
		/// Construct a finite state automaton transition function for a pattern.
		/// </summary>
		/// <param name="templatelength">The length of the pattern.</param>
		/// <param name="patternVal">The pattern to use. This transition function packs to trinary. The only acceptable characters in the alphabet are 0, 1, and 2, where 2 is anything that isn't 0 or 1.</param>
		/// <returns></returns>
		public byte[,] constructTransitionFunction( int templateLength, int patternVal )
		{
			patternVal ^= (1<<templateLength)-1;
			byte[,] ftrans = FSMLookup[templateLength][patternVal];
			if( ftrans != null )
			{
				return ftrans;
			}
			ftrans = new byte[templateLength,3];
			// construct transition "function"
			for( byte q = 0; q < templateLength; ++q )
			{
				// for each "character" in the pattern...
				//
				for( byte a = 0; a < 3; ++a )
				{
					// for each "character" in the alphabet
					//
					byte k = (byte)Math.Min(templateLength+1,q+2);
					bool pkIsSuffixOfPqA;
					do 
					{
						--k;
						int PqAoffset = (q+1)-k;
						pkIsSuffixOfPqA = true;
						for( int j = 0; j < k-1; ++j )
						{
							// check	:	Pk		from #0 to #(k-1)
							// against	:	Pq		from #PqAOffset to #(q-1)
							//if( complement[j] != complement[PqAoffset+j] )
							if( ((patternVal>>j)&1) != ((patternVal>>(PqAoffset+j))&1) )
							{
								pkIsSuffixOfPqA = false;
								break;
							}
						}
						// now test P[k-1] against A
						//if( pkIsSuffixOfPqA && k != 0 && complement[k-1] != a )
						if( pkIsSuffixOfPqA && k != 0 && ((patternVal>>(k-1))&1) != a )
						{
							pkIsSuffixOfPqA = false;
						}
					}
					while( ! pkIsSuffixOfPqA );
					ftrans[q,a] = k;
				}
			}
			// end construct transition "function"
			FSMLookup[templateLength][patternVal] = ftrans;
			return ftrans;
		}


		/// <summary>
		/// Find the complement of the template at curErgot.ip+1 - search only forward.
		/// </summary>
		/// <param name="curErgot">The creature doing the searching.</param>
		/// <param name="jumpOrAXCX">true:Set the creature's ip to 1 after the found complement. false: set ax to 1 after the found complement.</param>
		/// <param name="pushIP">push the creature's previous ip onto its stack when found.</param>
		public void doForwardADDR( ergot curErgot, bool jumpOrAXCX, bool pushIP )
		{
			// get the absolute position of the template in the soup.
			int templatePosition = ( curErgot.ip + curErgot.pos + sizeplusone )&modu;
			// templates can only be 8 instrs long.
			int templateStop = (templatePosition+8)&modu;
			int templateLength = 0;
			int fi;
			for( fi = templatePosition; fi != templateStop; fi = (fi+1)&modu )
			{
				// here we find out the length of the template
				if( soup[fi] > 1 ) { break; }
				++templateLength;
			}
			if( templateLength == 0 )
			{
				// no template!
				// set flag and stop.
				curErgot.fl = true;
				return;
			}
			// Construct the complement for the pattern.
			//byte[] complement = new byte[templateLength];
			//System.Text.StringBuilder fsb = new System.Text.StringBuilder(templateLength,templateLength);
			int f_val = 0;
			for( int i = 0; i < templateLength; ++i )
			{
				//complement[i] = soup[(templatePosition+i)&modu] == 1 ? (byte)0 : (byte)1;
				//fsb.Append(complement[i]);
				f_val ^= soup[(templatePosition+i)&modu] << i;
			}
			//
			
			// Construct forward transition functions for the pattern.
			byte[,] ftrans = constructTransitionFunction( templateLength, f_val );

			// Perform the search, forward only.
			short complementOffset = 0;
			short count = 0;
			byte fcur;
			byte fstate = 0;
			bool patternFound = false;
			for( /* fi is already 1 after the template */; count < 1024; fi = (fi+1)&modu, ++count )
			{
				fcur = soup[fi] > (byte)1 ? (byte)2 : soup[fi];
				fstate = ftrans[fstate, fcur];
				if( fstate == templateLength )
				{
					// We found it!
					patternFound = true;
					complementOffset = (short)(curErgot.ip + templateLength + 2 + count);
					break;
				}
			}
			//	Set flag and stop here if no match found.
			if( ! patternFound ) { curErgot.fl = true; return; }

			if( pushIP )
			{
				// push the current ip onto the stack
				curErgot.st[curErgot.sp] = curErgot.ip;
				curErgot.sp = (byte)((curErgot.sp+1) % 10);
			}
			if( jumpOrAXCX )
			{
				// set ip to the offset of the complement's end.
				curErgot.ip = (short)(complementOffset - 1);
			}
			else
			{
				// set ax to the offset of the complement's end.
				curErgot.ax = complementOffset;
				// set cx to the complement's length.
				curErgot.cx = (byte)templateLength;
			}
		}


		/// <summary>
		/// Find the complement of the template at curErgot.ip+1 - search only backward.
		/// </summary>
		/// <param name="curErgot">The creature doing the searching.</param>
		/// <param name="jumpOrAXCX">true:Set the creature's ip to 1 after the found complement. false: set ax to 1 after the found complement.</param>
		/// <param name="pushIP">push the creature's previous ip onto its stack when found.</param>
		public void doBackwardADDR( ergot curErgot, bool jumpOrAXCX, bool pushIP )
		{
			// get the absolute position of the template in the soup.
			int templatePosition = ( curErgot.ip + curErgot.pos + sizeplusone )&modu;
			// templates can only be 8 instrs long.
			int templateStop = (templatePosition+8)&modu;
			int templateLength = 0;
			for( int fi = templatePosition; fi != templateStop; fi = (fi+1)&modu )
			{
				// here we find out the length of the template
				if( soup[fi] > 1 ) { break; }
				++templateLength;
			}
			if( templateLength == 0 )
			{
				// no template!
				// set flag and stop.
				curErgot.fl = true;
				return;
			}
			// Construct the backlement for the pattern.
			//byte[] backlement = new byte[templateLength];
			//System.Text.StringBuilder bsb = new System.Text.StringBuilder(templateLength,templateLength);
			int templateIndex = (templatePosition+templateLength-1)&modu;
			int sizeLessOne = size - 1;
			int b_val = 0;
			for( int i = 0; i < templateLength; ++i )
			{
				//backlement[i] = soup[templateIndex] == 1 ? (byte)0 : (byte)1;
				//bsb.Append( backlement[i] );
				b_val ^= soup[templateIndex] << i;
				templateIndex = (templateIndex+sizeLessOne)&modu;
			}
			//
			
			// Construct backward transition functions for the pattern.
			byte[,] btrans = constructTransitionFunction( templateLength, b_val );

			// Perform the search, backward only.
			short complementOffset = 0;
			short count = 0;
			byte bcur;
			byte bstate = 0;
			bool patternFound = false;
			for( int bi = (templatePosition+sizeLessOne)&modu; count < 1024; bi = (bi+sizeLessOne)&modu, ++count )
			{
				bcur = soup[bi] > (byte)1 ? (byte)2 : soup[bi];
				bstate = btrans[bstate, bcur];
				if( bstate == templateLength )
				{
					// We found it!
					patternFound = true;
					complementOffset = (short)((curErgot.ip+templateLength) - count);
					break;
				}
			}
			//	Set flag and stop here if no match found.
			if( ! patternFound ) { curErgot.fl = true; return; }

			if( pushIP )
			{
				// push the current ip onto the stack
				curErgot.st[curErgot.sp] = curErgot.ip;
				curErgot.sp = (byte)((curErgot.sp+1) % 10);
			}
			if( jumpOrAXCX )
			{
				// set ip to the offset of the complement's end.
				curErgot.ip = (short)(complementOffset - 1);
			}
			else
			{
				// set ax to the offset of the complement's end.
				curErgot.ax = complementOffset;
				// set cx to the complement's length.
				curErgot.cx = (byte)templateLength;
			}
		}

		
		/// <summary>
		/// Find the complement of the template at curErgot.ip+1 - search forward AND backward simultaneously.
		/// </summary>
		/// <param name="curErgot">The creature doing the searching.</param>
		/// <param name="jumpOrAXCX">true:Set the creature's ip to 1 after the found complement. false: set ax to 1 after the found complement.</param>
		/// <param name="pushIP">push the creature's previous ip onto its stack when found.</param>
		public void doTwoWayADDR( ergot curErgot, bool jumpOrAXCX, bool pushIP )
		{
			// get the absolute position of the template in the soup.
			int templatePosition = ( curErgot.ip + curErgot.pos + sizeplusone )&modu;
			// templates can only be 8 instrs long.
			int templateStop = (templatePosition+8)&modu;
			int templateLength = 0;
			int fi;
			for( fi = templatePosition; fi != templateStop; fi = (fi+1)&modu )
			{
				// here we find out the length of the template
				if( soup[fi] > 1 ) { break; }
				++templateLength;
			}
			if( templateLength == 0 )
			{
				// no template!
				// set flag and stop.
				curErgot.fl = true;
				return;
			}
			// Construct the complement and backlement for the pattern.
			byte[] complement = new byte[templateLength];
			byte[] backlement = new byte[templateLength];
			
			int templateIndex = (templatePosition+templateLength-1)&modu;
			int sizeLessOne = size - 1;
			int f_val = 0;
			int b_val = 0;
			for( int i = 0; i < templateLength; ++i )
			{
				//complement[i] = soup[(templatePosition+i)&modu] == 1 ? (byte)0 : (byte)1;
				//fsb.Append( complement[i] );
				f_val ^= soup[(templatePosition+i)&modu] << i;
				//
				//backlement[i] = soup[templateIndex] == 1 ? (byte)0 : (byte)1;
				//bsb.Append( backlement[i] );
				b_val ^= soup[templateIndex] << i;
				templateIndex = (templateIndex+sizeLessOne)&modu;
			}
			//
			
			// Construct both transition functions for the pattern.
			byte[,] ftrans = constructTransitionFunction( templateLength, f_val );
			byte[,] btrans = constructTransitionFunction( templateLength, b_val );

			// Perform the search, forward and backward simultaneously.
			short complementOffset = 0;
			short count = 0;
			byte fcur, bcur;
			byte fstate = 0;
			byte bstate = 0;
			bool patternFound = false;
			for( int bi = (templatePosition+sizeLessOne)&modu; count < 1024;	//
				// fi is already 1 after the template					// VVV increments VVV
				fi = (fi+1)&modu,
				bi = (bi+sizeLessOne)&modu,
				++count )
			{
				fcur = soup[fi] > (byte)1 ? (byte)2 : soup[fi];
				fstate = ftrans[fstate, fcur];
				if( fstate == templateLength )
				{
					// We found it!
					patternFound = true;
					complementOffset = (short)(curErgot.ip + templateLength + 2 + count);
					break;
				}
				bcur = soup[bi] > (byte)1 ? (byte)2 : soup[bi];
				bstate = btrans[bstate, bcur];
				if( bstate == templateLength )
				{
					// We found it!
					patternFound = true;
					complementOffset = (short)((curErgot.ip+templateLength) - count);
					break;
				}
			}
			//	Set flag and stop here if no match found.
			if( ! patternFound ) { curErgot.fl = true; return; }

			if( pushIP )
			{
				// push the current ip onto the stack
				curErgot.st[curErgot.sp] = curErgot.ip;
				curErgot.sp = (byte)((curErgot.sp+1) % 10);
			}
			if( jumpOrAXCX )
			{
				// set ip to the offset of the complement's end.
				curErgot.ip = (short)(complementOffset - 1);
			}
			else
			{
				// set ax to the offset of the complement's end.
				curErgot.ax = complementOffset;
				// set cx to the complement's length.
				curErgot.cx = (byte)templateLength;
			}
		}
		#endregion

		#region find empty space
		/// <summary>
		/// Find the nearest empty space within 65536 cells of the specified position.
		/// It searches forward first and then wraps around and searches backward.
		/// It returns null on failure, or a new (and not yet living) ergot if found,
		/// with the new ergot's size and position set to the position found.
		/// </summary>
		/// <param name="positionOfParent">Position of the parent.</param>
		/// <param name="desiredOffset">Desired offset from the parent at which to start searching.</param>
		/// <param name="length">The size of the empty space to find.</param>
		/// <returns>null, or a new ergot if found.</returns>
		public ergot findEmptySpace( int positionOfParent, short desiredOffset, short length )
		{
			if( length < 1 )
			{
				return null;
			}

			// We must find a suitable run of empty cells for the new ergot.
			
			ergot adam;
			
			int index;
			int childPosition;
			int posOfParentPlusSize = positionOfParent + size;

			short childOffset;
			short zerosLeft = length;
			short lengthPlusOne = (short)(length + 1);
			
			for( childOffset = desiredOffset; childOffset != Machine.c_rel_min && zerosLeft != 0; ++childOffset )
			{
				index = (posOfParentPlusSize+childOffset)&modu;
				if( usage[index] != Machine.c_cell_dead )
				{
					zerosLeft = lengthPlusOne;
				}
				--zerosLeft;
			}

			if( zerosLeft == 0 )
			{
				// hey, the first search area returned a zero-run of size 'length' - yay!
				childOffset -= length;
				childPosition = (posOfParentPlusSize+childOffset)&modu;
			}
			else
			{
				// we're gonna do a backwards search now
				// (comparing against a constant is faster than stopping at a specific position)
				zerosLeft = length;
				for( childOffset = desiredOffset; childOffset != Machine.c_rel_max && zerosLeft != 0; )
				{
					--childOffset;
					index = (posOfParentPlusSize+childOffset)&modu;
					if( usage[index] != Machine.c_cell_dead )
					{
						zerosLeft = lengthPlusOne;
					}
					--zerosLeft;
				}
				if( zerosLeft == 0 )
				{
					childPosition = (posOfParentPlusSize+childOffset)&modu;
				}
				else
				{
					return null;
				}
			}

			// DE/BUG: Find out if there is any overlapping of creature address spaces...
			//			for( short i = 0; i < length; ++i )
			//			{
			//				index = (childPosition+i)&modu;
			//				if( usage[index] != Machine.c_cell_dead )
			//				{
			//					throw new Exception("creature overlap! ");
			//				}
			//			}

			if( resurrectionStack_TopIndex == -1 )
			{
				// nothing in the resurrectionStack. We must create a new ergot.
				adam = new ergot( this, length, childPosition );
			}
			else
			{
				// we can re-use an ergot from the resurrectionStack.
				adam = resurrectionStack[resurrectionStack_TopIndex];
				resurrectionStack[resurrectionStack_TopIndex] = null;
				--resurrectionStack_TopIndex;
				adam.reUse( length, childPosition );
			}
			adam.ax = childOffset;
			return adam;
		}
		#endregion

		#region graphics related

		#region coloring
		/// <summary>
		/// Set the cell shade and dead shade for a given instruction at index i.
		/// </summary>
		/// <param name="i">The decimal index of the instruction.</param>
		/// <param name="red">integer 0 to 255.</param>
		/// <param name="green">integer 0 to 255.</param>
		/// <param name="blue">integer 0 to 255.</param>
		private void initializeShade( int i, byte red, byte green, byte blue )
		{
			cellshades[i,2] = red;
			cellshades[i,1] = green;
			cellshades[i,0] = blue;
			deadshades[i,2] = (byte)(red*0.6);
			deadshades[i,1] = (byte)(green*0.6);
			deadshades[i,0] = (byte)(blue*0.6);
		}

		/// <summary>
		/// Initialize the cell shades for every instruction. Called automatically by loadDefaults().
		/// </summary>
		private void initializeCellShades()
		{
			cellshades = new byte[c_num_instrs,3];
			deadshades = new byte[c_num_instrs,3];

			for( int i = 0x00; i < Machine.c_num_instrs; ++i )
			{
				// set a default color for everything just to make
				// sure that we don't have anything black (except nop_0)
				initializeShade(i, (byte)(i), (byte)(100 + (i << 1)), (byte)(240 - (i << 1)));
				//initializeShade(i, 0, 175, 188);
			}

			// customized colors follow...

			initializeShade( 0x00, 0, 0, 0 );
			initializeShade( 0x01, 240, 240, 240 );
//			initializeShade( 0x02, 255, 199, 255 );
//			initializeShade( 0x03, 255, 217, 255 );
//			initializeShade( 0x04, 255, 160, 255 );
//			initializeShade( 0x05, 255, 185, 255 );
//			initializeShade( 0x06, 255, 160, 10 );
//			initializeShade( 0x07, 255, 182, 72 );
//			initializeShade( 0x08, 0, 255, 255 );
//			initializeShade( 0x09, 0, 250, 230 );
//			initializeShade( 0x0A, 155, 255, 255 );
//			initializeShade( 0x0B, 182, 255, 255 );
//			initializeShade( 0x0C, 0, 255, 0 );
//			initializeShade( 0x0D, 113, 255, 117 );
//			initializeShade( 0x0E, 0, 255, 153 );
//			initializeShade( 0x0F, 163, 255, 170 );
//			initializeShade( 0x10, 255, 95, 255 );
//			initializeShade( 0x11, 255, 104, 230 );
//			initializeShade( 0x12, 255, 30, 255 );
//			initializeShade( 0x13, 186, 0, 222 );
//			initializeShade( 0x14, 0, 200, 255 );
//			initializeShade( 0x15, 37, 185, 255 );
//			initializeShade( 0x16, 0, 157, 255 );
//			initializeShade( 0x17, 0, 200, 255 );
//			initializeShade( 0x18, 255, 255, 0 );
//			initializeShade( 0x19, 255, 235, 20 );
//			initializeShade( 0x1A, 255, 210, 0 );
//			initializeShade( 0x1B, 255, 200, 154 );
//			initializeShade( 0x1C, 255, 245, 165 );
//			initializeShade( 0x1D, 255, 219, 175 );
//			initializeShade( 0x1E, 255, 0, 0 );
//			initializeShade( 0x1F, 255, 15, 150 );
		}
		#endregion

		/// <summary>
		/// Update the soup bitmap. Prints different colors for living and dead cells.
		/// Also prints the instruction pointers for all creatures as tiny pink boxes.
		/// </summary>
		unsafe public void updateSoupBitmap()
		{
			if( soupDisplayMode == 1 )
			{
				updateCreatureBitmap();
				return;
			}
			System.Drawing.Imaging.BitmapData bmd = sbmp.LockBits(soupRect,
				System.Drawing.Imaging.ImageLockMode.ReadOnly, sbmp.PixelFormat);

			int x, y, thePixel, instrCode;
			int index = 0;
			for( y = 0; y < height; ++y )
			{
				byte* row=(byte *)bmd.Scan0+(y*bmd.Stride);
				for( x = 0; x < width; ++x )
				{
					thePixel = (x << 1) + x;						// thePixel = x * 3;
					instrCode = soup[index] & (c_num_instrs-1);	// fast-modulus by c_num_instrs
					if( usage[index] == Machine.c_cell_dead )
					{
						row[thePixel]= deadshades[instrCode,0];
						row[thePixel+1]= deadshades[instrCode,1];
						row[thePixel+2]= deadshades[instrCode,2];
					}
					else
					{
						row[thePixel]= cellshades[instrCode,0];
						row[thePixel+1]= cellshades[instrCode,1];
						row[thePixel+2]= cellshades[instrCode,2];
					}
					++index;
				}
			}
			sbmp.UnlockBits(bmd);
			
			if( ! ergotList.isEmpty )
			{
				ergot curErgot;
				int ipTop, ipLeft;
				int fastwidthmod = width - 1;
				System.Drawing.Graphics g = Graphics.FromImage(sbmp);
				KTL.Node cur = ergotList.Head;
				while( ! cur.isNull )
				{
					curErgot = (ergot)cur.Value;
					// fast divide by height
					ipTop = ((curErgot.ip + curErgot.pos) >> opSoupSizeRoot) - 1;
					// fast modulus by width
					ipLeft = ((curErgot.ip + curErgot.pos) & fastwidthmod) - 1;
					if( curErgot.ip >= 0 && curErgot.ip < curErgot.mySize )
					{
						g.DrawRectangle( p_internal, ipLeft, ipTop, 2, 2 );
					}
					else
					{
						g.DrawRectangle( p_external, ipLeft, ipTop, 2, 2 );
					}
					cur = cur.NextNode;
				}
				g.Dispose();
			}
		}

		
		/// <summary>
		/// Update the soup bitmap. Prints different colors for adults and children.
		/// Also prints the instruction pointers for all creatures as tiny pink boxes.
		/// </summary>
		unsafe public void updateCreatureBitmap()
		{
			System.Drawing.Imaging.BitmapData bmd = sbmp.LockBits(soupRect,
				System.Drawing.Imaging.ImageLockMode.ReadOnly, sbmp.PixelFormat);

			int x, y, thePixel, instrCode;
			int index = 0;
			for( y = 0; y < height; ++y )
			{
				byte* row=(byte *)bmd.Scan0+(y*bmd.Stride);
				for( x = 0; x < width; ++x )
				{
					thePixel = (x << 1) + x;						// thePixel = x * 3;
					instrCode = soup[index] & (c_num_instrs-1);	// fast-modulus by c_num_instrs
					if( usage[index] >= Machine.c_cell_embryo )
					{
						// embryo
						row[thePixel]= 0;
						row[thePixel+1]= 0;
						row[thePixel+2]= 200;
					}
					else if( usage[index] != Machine.c_cell_dead )
					{
						// adult
						if( istt[instrCode] )
						{
							// DEBUG: Show the old instructions!
							row[thePixel]= 200;
							row[thePixel+1]= 0;
							row[thePixel+2]= 0;
						}
						else
						{
							// DEBUG: Show the new instructions!
							row[thePixel]= 0;
							row[thePixel+1]= 200;
							row[thePixel+2]= 0;
						}
					}
					else
					{
						// dead/unused
						row[thePixel]= deadshades[instrCode,0];
						row[thePixel+1]= deadshades[instrCode,1];
						row[thePixel+2]= deadshades[instrCode,2];
					}
					++index;
				}
			}
			sbmp.UnlockBits(bmd);
			if( ! ergotList.isEmpty )
			{
				ergot curErgot;
				int ipTop, ipLeft;
				int fastwidthmod = width - 1;
				System.Drawing.Graphics g = Graphics.FromImage(sbmp);
				KTL.Node cur = ergotList.Head;
				while( ! cur.isNull )
				{
					curErgot = (ergot)cur.Value;
					// fast divide by height
					ipTop = ((curErgot.ip + curErgot.pos) >> opSoupSizeRoot) - 1;
					// fast modulus by width
					ipLeft = ((curErgot.ip + curErgot.pos) & fastwidthmod) - 1;
					if( curErgot.ip >= 0 && curErgot.ip < curErgot.mySize )
					{
						g.DrawRectangle( p_internal, ipLeft, ipTop, 2, 2 );
					}
					else
					{
						g.DrawRectangle( p_external, ipLeft, ipTop, 2, 2 );
					}
					cur = cur.NextNode;
				}
				g.Dispose();
			}
		}
		#endregion

		#region ioError
		public void ioError( string theName, Exception e )
		{
			_errorString.Append( "XML Input encountered an error at " + theName + ": " + e.Message +"\n" );
			_errorCode = 10;
		}
		public void saveError( string theName, Exception e )
		{
			_errorString.Append( "XML Output encountered an error at " + theName + ": " + e.Message +"\n" );
			_errorCode = -10;
		}
		#endregion

		#region FromXMLFile
		public void FromXMLFile( string xmlFileName )
		{
			if( isCycling )
			{
				// don't do anything if a thread is running me.
				return;
			}
			XmlTextReader x = null;
			try
			{
				_errorCode = 0;
				_errorString = new System.Text.StringBuilder(2001);
				x = new XmlTextReader(xmlFileName);
				while( x.Name == "" )
				{
					x.Read();
				}

				// save the file name for posterity.
				_xmlFileName = xmlFileName;

				if( x.Name.ToLower() != "xml" )
				{
					// not an XML document.
					_errorCode = 2;
					_errorString.Append( "File " + xmlFileName + " was not a valid XML file.\n" );
					return;
				}
				while( x.Read() )
				{
					if( x.NodeType == XmlNodeType.Element && x.Name.ToLower() == "machine" )
					{
						break;
					}
				}
				if( x.EOF )
				{
					// not the correct type of XML document.
					_errorCode = 2;
					_errorString.Append( "File " + xmlFileName + " was empty, or was not a valid XML::Machine file.\n" );
					return;
				}

				while( x.Read() )
				{
					switch(x.NodeType)
					{
						case XmlNodeType.Element:
						switch(x.Name.ToLower())
						{
							case "variables":
								// call load var func
								VariablesFromXML( x );
								break;
							case "options":
								// call load options func
								OptionsFromXML( x );
								initialize();
								break;
							case "genebank":
								// load genebank
								this.myGeneBank.FromXMLStream( x, this );
								break;
							case "creatures":
								// loop and create creatures with XML stream as arg to constructor
								CreaturesFromXML( x );
								break;
							default:
								break;
						}
							break;
						case XmlNodeType.Text:
							break;
						case XmlNodeType.EndElement:
							break;
						default:
							break;
					}
				}
				x.Close();
			}
			catch (Exception ex)
			{
				ioError(xmlFileName, ex);
				try
				{
					if( x != null )
					{
						x.Close();
					}
				}
				catch
				{
					;
				}
			}
		}

		#region CreaturesFromXML
		public void CreaturesFromXML( XmlTextReader x )
		{
			while( x.Name == "" )
			{
				x.Read();
			}
			if( x.Name.ToLower() != "creatures" )
			{
				// not the correct type of grouping.
				return;
			}
			string nodeName = "";

			while( x.Read() )
			{
				switch (x.NodeType)
				{
					case XmlNodeType.Element:
						nodeName = x.Name.ToLower();
					switch (nodeName)
					{
						case "ergot":
							ergot curErgot = new ergot(x, this);
							break;
						default:
							break;
					}
						break;
					case XmlNodeType.Text:
						break;
					case XmlNodeType.EndElement:
						if( nodeName == "creatures" )
						{
							return;
						}
						nodeName = "";
						break;
					default:
						break;
				} // switch
			} // while
		} // function
		#endregion

		#region VariablesFromXML
		private void VariablesFromXML( XmlTextReader x )
		{
			while( x.Name == "" )
			{
				x.Read();
			}
			if( x.Name.ToLower() != "variables" )
			{
				// not the correct type of grouping.
				return;
			}
			string nodeName = "";
			int savedSize = 0;

			while( x.Read() )
			{
				switch (x.NodeType)
				{
					case XmlNodeType.Element:
						nodeName = x.Name.ToLower();
					switch (nodeName)
					{
						case "wire":
							try
							{
								int binLength = 0;
								binLength = System.Int32.Parse( x.GetAttribute("length"), intStyle );
								if( binLength > 0 )
								{
									byte[] wirebin = new byte[binLength];
									x.ReadBinHex(wirebin, 0, binLength);
									System.IO.MemoryStream ms = new System.IO.MemoryStream();
									System.Runtime.Serialization.IFormatter ff = new System.Runtime.Serialization.Formatters.Binary.BinaryFormatter();
									ms.Write(wirebin, 0, binLength);
									ms.Seek(0, System.IO.SeekOrigin.Begin);
									wire = (System.Random)ff.Deserialize(ms);
								}
							}
							catch (Exception e)
							{
								ioError(nodeName, e);
							}
							break;
						case "soup":
							try
							{
								x.ReadBinHex(soup, 0, savedSize);
							}
							catch (Exception e)
							{
								ioError(nodeName, e);
							}
							break;
						default:
							break;
					}
						break;
					case XmlNodeType.Text:
					switch (nodeName)
					{
						case "slicevary":
							try
							{
								sliceVary = new int[Machine.c_slicevary_size];
								string tmp1 = System.Text.RegularExpressions.Regex.Replace( x.Value, "\\s+", "" );
								string[] tmp = tmp1.Split(",".ToCharArray(),Machine.c_slicevary_size);
								for( int i = 0; i < Machine.c_slicevary_size; ++i )
								{
									sliceVary[i] = System.Int32.Parse( tmp[i], intStyle );
								}
							}
							catch (Exception e)
							{
								ioError(nodeName,e);
							}
							break;
						case "numticks":
							try
							{
								numTicks = System.UInt64.Parse( x.Value, intStyle );
							}
							catch (Exception e)
							{
								ioError(nodeName,e);
							}
							break;
						case "numcycles":
							try
							{
								numCycles = System.UInt64.Parse( x.Value, intStyle );
							}
							catch (Exception e)
							{
								ioError(nodeName,e);
							}
							break;
						case "numspeciessighted":
							try
							{
								numSpeciesSighted = System.UInt32.Parse( x.Value, intStyle );
							}
							catch (Exception e)
							{
								ioError(nodeName,e);
							}
							break;
						case "soupdisplaymode":
							try
							{
								soupDisplayMode = System.Byte.Parse( x.Value, bytStyle );
							}
							catch (Exception e)
							{
								ioError(nodeName,e);
							}
							break;
						case "size":
							try
							{
								savedSize = System.Int32.Parse( x.Value, intStyle );
							}
							catch (Exception e)
							{
								ioError(nodeName,e);
							}
							break;
						case "usage":
							try
							{
								//ma.usage = new Byte[ma.size];
								string tmp = x.Value;
								for( int i = 0; i < savedSize && i < tmp.Length; ++i )
								{
									usage[i] = (byte)(tmp[i]-48);
								}
							}
							catch (Exception e)
							{
								ioError(nodeName,e);
							}
							break;
						default:
							break;
					}
						break;
					case XmlNodeType.EndElement:
						if( x.Name.ToLower() == "variables" )
						{
							return;
						}
						nodeName = "";
						break;
					default:
						break;
				} // switch
			} // while
		} // VariablesFromXML
		#endregion

		#region OptionsFromXML
		private void OptionsFromXML( XmlTextReader x )
		{
			while( x.Name == "" )
			{
				x.Read();
			}
			if( x.Name.ToLower() != "options" )
			{
				// not the correct type of grouping.
				return;
			}
			loadDefaults();
			string nodeName = "";
			
			while( x.Read() )
			{
				switch (x.NodeType)
				{
					case XmlNodeType.Element:
						nodeName = x.Name.ToLower();
						break;
					case XmlNodeType.Text:
					switch (nodeName)
					{
						case "opdebug":
							try
							{
								opDebug = System.Boolean.Parse( x.Value );
							}
							catch (Exception e)
							{
								ioError(nodeName,e);
							}
							break;
						case "opsoupsizeroot":
							try
							{
								opSoupSizeRoot = System.Int32.Parse( x.Value, intStyle );
							}
							catch (Exception e)
							{
								ioError(nodeName,e);
							}
							break;
						case "opsoupslicetime":
							try
							{
								opSoupSliceTime = System.Int32.Parse( x.Value, intStyle );
							}
							catch (Exception e)
							{
								ioError(nodeName,e);
							}
							break;
						case "opsoupsaveautointerval":
							try
							{
								opSoupSaveAutoInterval = System.Int32.Parse( x.Value, intStyle );
							}
							catch (Exception e)
							{
								ioError(nodeName,e);
							}
							break;
						case "opsoupsliceconstant":
							try
							{
								opSoupSliceConstant = System.Boolean.Parse( x.Value );
							}
							catch (Exception e)
							{
								ioError(nodeName,e);
							}
							break;
						case "opsoupsliceleanbias":
							try
							{
								opSoupSliceLeanBias = System.Boolean.Parse( x.Value );
							}
							catch (Exception e)
							{
								ioError(nodeName,e);
							}
							break;
						case "opsoupsaveautoasnew":
							try
							{
								opSoupSaveAutoAsNew = System.Boolean.Parse( x.Value );
							}
							catch (Exception e)
							{
								ioError(nodeName,e);
							}
							break;
						case "opsoupslicefunctionofsize":
							try
							{
								opSoupSliceFunctionOfSize = System.Double.Parse( x.Value, dblStyle );
							}
							catch (Exception e)
							{
								ioError(nodeName,e);
							}
							break;
						case "opsoupwritepermissions":
							try
							{
								opSoupWritePermissions = (MachineOptions.WritePermissions)(System.Enum.Parse( typeof(MachineOptions.WritePermissions), x.Value, true ));
								//opSoupWritePermissions = System.Byte.Parse( x.Value, bytStyle );
							}
							catch (Exception e)
							{
								ioError(nodeName,e);
							}
							break;
						case "opreapertrigger":
							try
							{
								opReaperTrigger = System.Double.Parse( x.Value, dblStyle );
							}
							catch (Exception e)
							{
								ioError(nodeName,e);
							}
							break;
						case "opreapererase":
							try
							{
								opReaperErase = System.Boolean.Parse( x.Value );
							}
							catch (Exception e)
							{
								ioError(nodeName,e);
							}
							break;
						case "opreaperlog":
							try
							{
								opReaperLog = System.Boolean.Parse( x.Value );
							}
							catch (Exception e)
							{
								ioError(nodeName,e);
							}
							break;
						case "opmallocation":
							try
							{
								opMallocation = (MachineOptions.AllocationAddress)(System.Enum.Parse( typeof(MachineOptions.AllocationAddress), x.Value, true ));
								//opMallocation = System.Byte.Parse( x.Value, bytStyle );
							}
							catch (Exception e)
							{
								ioError(nodeName,e);
							}
							break;
						case "opmallocerase":
							try
							{
								opMallocErase = System.Boolean.Parse( x.Value );
							}
							catch (Exception e)
							{
								ioError(nodeName,e);
							}
							break;
						case "opforkcopyregisters":
							try
							{
								opForkCopyRegisters = System.Boolean.Parse( x.Value );
							}
							catch (Exception e)
							{
								ioError(nodeName,e);
							}
							break;
						case "operrslice":
							try
							{
								opErrSlice = System.Double.Parse( x.Value, dblStyle );
							}
							catch (Exception e)
							{
								ioError(nodeName,e);
							}
							break;
						case "operrcosmic":
							try
							{
								opErrCosmic = System.Double.Parse( x.Value, dblStyle );
							}
							catch (Exception e)
							{
								ioError(nodeName,e);
							}
							break;
						case "operrcopy":
							try
							{
								opErrCopy = System.Double.Parse( x.Value, dblStyle );
							}
							catch (Exception e)
							{
								ioError(nodeName,e);
							}
							break;
						case "operrflaw":
							try
							{
								opErrFlaw = System.Double.Parse( x.Value, dblStyle );
							}
							catch (Exception e)
							{
								ioError(nodeName,e);
							}
							break;
						case "operrtype":
							try
							{
								opErrType = (MachineOptions.ErrorType)(System.Enum.Parse( typeof(MachineOptions.ErrorType), x.Value, true ));
								//opErrType = System.Byte.Parse( x.Value, bytStyle );
							}
							catch (Exception e)
							{
								ioError(nodeName,e);
							}
							break;
						case "operrflaginstr":
							try
							{
								opErrFlagInstr = System.Boolean.Parse( x.Value );
							}
							catch (Exception e)
							{
								ioError(nodeName,e);
							}
							break;
						default:
							break;
					}
						break;
					case XmlNodeType.EndElement:
						if( x.Name.ToLower() == "options" )
						{
							return;
						}
						nodeName = "";
						break;
					default:
						break;
				} // switch
			} // while
		} // OptionsFromXML
		#endregion

		#endregion

		#region ToXMLFile	
		/// <summary>
		/// Output this Machine's settings and all the living creatures to the specified file.
		/// </summary>
		/// <param name="xmlFileName">The file name to use. This file will be overwritten if it exists.</param>
		public void ToXMLFile( string xmlFileName )
		{
			if( isCycling )
			{
				// don't do anything if a thread is running me.
				return;
			}
			XmlTextWriter x = null;

			try
			{
				x = new XmlTextWriter(xmlFileName, null);
				x.Formatting = System.Xml.Formatting.Indented;
				x.WriteStartDocument();

				x.WriteStartElement( "Machine" );
				if(true)
				{
					//
					// User-modifiable settings:
					//
					x.WriteStartElement("options");
					if(true)
					{
						//soup:
						x.WriteElementString("opSoupSizeRoot", opSoupSizeRoot.ToString());
						x.WriteElementString("opSoupSliceTime", opSoupSliceTime.ToString());
						x.WriteElementString("opSoupSaveAutoInterval", opSoupSaveAutoInterval.ToString());
						x.WriteElementString("opSoupSliceConstant", opSoupSliceConstant.ToString());
						x.WriteElementString("opSoupSliceLeanBias", opSoupSliceLeanBias.ToString());
						x.WriteElementString("opSoupSliceFunctionOfSize", opSoupSliceFunctionOfSize.ToString());
						x.WriteElementString("opSoupWritePermissions", opSoupWritePermissions.ToString());
						//		internal int opSoupWidth, opSoupHeight;
						//		internal int opSoupSliceTime, opSoupSaveAutoInterval;
						//		internal bool opSoupSliceConstant, opSoupSliceLeanBias, opSoupSaveAutoAsNew;
						//		internal double opSoupSliceFunctionOfSize;
						//		internal byte opSoupWritePermissions;	// 0: child and unused, 1: child, 2: cousin, 3: global

						// reaper:
						x.WriteElementString("opReaperTrigger", opReaperTrigger.ToString());
						x.WriteElementString("opReaperErase", opReaperErase.ToString());
						x.WriteElementString("opReaperLog", opReaperLog.ToString());
						//		internal double opReaperTrigger;
						//		internal bool opReaperErase, opReaperLog;

						// malloc/fork:
						x.WriteElementString("opMallocation", opMallocation.ToString());
						x.WriteElementString("opMallocErase", opMallocErase.ToString());
						x.WriteElementString("opForkCopyRegisters", opForkCopyRegisters.ToString());
						//		internal byte opMallocation;		// 0:ax, 1:parent, 2:random, 3:random packed
						//		internal bool opMallocErase, opForkCopyRegisters;
									
						// errors:
						x.WriteElementString("opDebug", opDebug.ToString());
						x.WriteElementString("opErrSlice", opErrSlice.ToString());
						x.WriteElementString("opErrCosmic", opErrCosmic.ToString());
						x.WriteElementString("opErrCopy", opErrCopy.ToString());
						x.WriteElementString("opErrFlaw", opErrFlaw.ToString());
						x.WriteElementString("opErrType", opErrType.ToString());
						x.WriteElementString("opErrFlagInstr", opErrFlagInstr.ToString());
						//		internal double opErrSlice, opErrCosmic, opErrCopy, opErrFlaw;
						//		internal byte opErrType;			// 0: bit, 1: adjacent, 2: random, 3: abs(st[sp] mod 32)
					} // options
					x.WriteEndElement();
					//
					// End User-modifiable settings.
					//

					
					x.WriteStartElement( "variables" );
					if(true)
					{
						x.WriteElementString( "size", size.ToString() );
						x.WriteElementString( "width", width.ToString() );
						x.WriteElementString( "height", height.ToString() );
						x.WriteElementString( "load", load.ToString() );
						x.WriteElementString( "numCycles", numCycles.ToString() );
						x.WriteElementString( "numTicks", numTicks.ToString() );
						x.WriteElementString( "numSpeciesSighted", numSpeciesSighted.ToString() );
						x.WriteElementString( "soupDisplayMode", soupDisplayMode.ToString() );
						
						if( wire != null )
						{
							System.IO.MemoryStream ms = new System.IO.MemoryStream();
							System.Runtime.Serialization.IFormatter ff = new System.Runtime.Serialization.Formatters.Binary.BinaryFormatter();
							ff.Serialize(ms, wire);
							ms.Seek(0, System.IO.SeekOrigin.Begin);
							byte[] wirebin = ms.ToArray();
							x.WriteStartElement("wire");
							if(true)
							{
								x.WriteAttributeString("length", wirebin.Length.ToString());
								x.WriteBinHex(wirebin, 0, wirebin.Length);
							}
							x.WriteEndElement();
							ms.Close();
						}

						if(true)
						{
							// save sliceVary
							x.WriteStartElement( "sliceVary" );
							if(true)
							{
								for( int i = 0; i < c_slicevary_size; ++i )
								{
									x.WriteString( this.sliceVary[i].ToString() );
									if( i < Machine.c_slicevary_size - 1 )
									{
										x.WriteString( "," );
									}
								}
							}
							x.WriteEndElement();
						}

						x.WriteStartElement("soup");
						if(true)
						{
							x.WriteBinHex( soup, 0, size );
						}
						x.WriteEndElement();
						x.WriteStartElement("usage");
						if(true)
						{
							for( int i = 0; i < size; ++i )
							{
								x.WriteString( usage[i].ToString() );
							}
						}
						x.WriteEndElement();
					} // end variables
					x.WriteEndElement();

					myGeneBank.ToXMLStream(x);


					if(true)
					{
						KTL.Node cur = reaperQueue.Head;
						int pos = 0;
						while( ! cur.isNull )
						{
							((ergot)cur.Value).reaperQueuePosition = pos;
							++pos;
							cur = cur.NextNode;
						}
					}


					x.WriteStartElement( "creatures" );
					if(true)
					{
						// write the list of creatures
						KTL.Node cur = ergotList.Head;
						while( ! cur.isNull )
						{
							((ergot)cur.Value).ToXMLStream(x);
							cur = cur.NextNode;
						}
					} // creatures
					x.WriteEndElement();
				} // Machine
				x.WriteEndElement();
				x.WriteEndDocument();
				x.Close();
			}
			catch (Exception ex)
			{
				saveError(xmlFileName, ex);
				try
				{
					if( x != null )
					{
						x.Close();
					}
				}
				catch
				{
					;
				}
			}
		}
		#endregion

		#region Modulus
		public int deprecated_wrap( int pos )
		{
			return( pos < 0 ? (pos+size) & modu : pos & modu );
		}
		#endregion

        
	}
}
